pub enum RecoveryError {
Eof,
AlreadyRecovered,
RecoveryDisabled,
}Variants
Eof
Recovery failed because the parser reached the end of file
AlreadyRecovered
Recovery failed because it didn’t eat any tokens. Meaning, the parser is already in a recovered state.
This is an error because:
a) It shouldn’t create a completed marker wrapping no tokens
b) This results in an infinite-loop if the recovery is used inside a while loop. For example,
it’s common that list parsing also recovers at the end of a statement or block. However, list elements
don’t start with a ; or } which is why parsing, for example, an array element fails again and
the array expression triggers another recovery. Handling this as an error ensures that list parsing
rules break out of the loop the same way as they would at the EOF.
RecoveryDisabled
Returned if there’s an unexpected token and the parser is speculatively parsing a syntax. Error-recovery is disabled when doing speculative parsing because it can then make the impression that the parser was able to correctly parse a syntax when, in fact, it only skipped over the tokens.
For example the syntax (a, b, c) ... in JavaScript can either be a parenthesized expression
or an arrow function, depending on what kind of token ... is. Thus, the parer’s only option is
to make the following assumption:
“Let’s assume (a, b, c) ...” is an arrow function expression“
The parser then tries to parse (a, b, c) as an arrow function parameters and validates that ...
indeed is the =>. The parser rewinds and re-parses the syntax as a parenthesized expression
if it turns out that ... isn’t the => token or if any element in (a, b, c) isn’t a valid parameter (for example 5 + 3 isn’t valid).